package ch.ethz.icr.tutorial.simpleipd;

/**
 * Simple Iterated Prisoners' Dilemma Model: The agent's code
 *
 * This is the first step of the tutorial sequence. This phase sets up a 2 x 2
 * iterated prisoner's dilemma game where the players can have one out of
 * our strategies.
 *
 * For the original model, see Cohen, Riolo, and Axelrod:
 * The Emergence of Social Organization in the Prisoner's Dilemma
 * (SFI Working Paper), 1999. http://www.santafe.edu/ (follow publications link).
 *
 * @author Luc Girardin
 * @author Lars-Erik Cederman
 * @author Laszlo Gulyas
 * @version 2.0
 */
public class Player {
    // Constants for actions
    static final int C = 1;             // Cooperate
    static final int D = 0;             // Defect
    // Conversion array from action to String
    static final String[] actionToString = {"D", "C"};

    // Action matrices
    // The x_PARAMS[] matrices define the behavior for each of the four strategies
    // (i.e., ALLC, TFT, ATFT, ALLD, respectively).
    public final int[] I_PARAMS = {C, C, D, D}; // First action
    public final int[] P_PARAMS = {C, C, D, D}; // Action if opponent cooperated
    public final int[] Q_PARAMS = {C, D, C, D}; // Action if opponent defected

    // The agent's internal variables
    Player other;                       // Handle to the opponent
    int type;                           // The agent's strategy
    int[][] prefs = {{1, 5}, // Payoff / Preference matrix
                     {0, 3}};          // (this one is PD)
    int action;                         // The current action
    int memory;                         // The opponent's last action
    int cumulPayoff;                    // The agent's cumulated payoff

    /**
     * Creating the agent
     *
     * @param t the strategy type
     */
    public Player(int t) {
        type = t;
    }

    /**
     * Storing the opponent's last action
     */
    public void remember() {
        memory = other.action;
    }

    /**
     * The agent's decision-making.
     * This is the main behavioral method that uses one of the x_PARAMS[][]
     * matrices depending on whether the move is the first one or the other
     * player cooperated. The actual move is also dependent on the agent's
     * type. The move is recorded in the player's action variable.
     *
     * @param time the current time
     */
    public void play(int time) {
        if (time == 1)
            action = I_PARAMS[type];
        else if (memory == C)
            action = P_PARAMS[type];
        else
            action = Q_PARAMS[type];
    }

    /**
     * Administering the payoff.
     * Updates the cumulative payoff by adding the payoffs from the preference
     * matrix as a function of both sides' moves.
     */
    public void addPayoff() {
        cumulPayoff = cumulPayoff + prefs[action][other.action];
    }

    /**
     * Converts the player's statistics into String format.
     * (Used when reporting the current moves & statistics.)
     *
     * @return a textual representation of the player
     */
    public String toString() {
        return "{" +
                actionToString[action] + "," +
                cumulPayoff +
                "}";
    }
}